@@ -1,10 +1,13 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Wed Dec  5 17:19:33 2018
-
 This module includes classes and functions relating string coloring and format.
 
-@author: Guest Group
 """
 from colorpalette import Color, Palette
 import numpy as np
